"use strict";

var error = require("./../error");

var HerokuCommandApps = module.exports = function(options) {
    this.options = options;
};

HerokuCommandApps.$options = {
    "name"   : true,
    "addons" : true,
    //"remote" : true,
    "stack"  : true,
    "timeout": true
};

(function() {
    
    this.list = this.apps = this.index = function(callback) {
        this.$client.list(callback);
    };
    
    this.info = function(callback) {
        if (this.options.name === "")
            throw new error.InvalidOption("The name or domain of the app "
            + "cannot be empty.");
            
        this.$client.appInfo(this.options.name, callback);
        // @todo
        //   - list collaborators
        //   - list addons
    };
    
    function createComplete(callback) {
        var self = this;
        var opts = this.options;
        
        var ti = setTimeout(function() {
            ti = null;
            callback(new error.Runtime("Timed Out! Check heroku info" +
                " for status updates."));
        }, opts.timeout);
        
        (function repeat() {
            if (!ti)
                return;
                
            self.$client.createComplete(opts.name, function(err, created, data) {
                if (!ti)
                    return;
                
                if (err || created) {
                    clearTimeout(ti);
                    if (err)
                        return callback(err);
                    
                    afterCreateComplete.call(self, data, callback);
                } else
                    setTimeout(repeat, 1000);
            });
        })();
    }
    
    function afterCreateComplete(data, callback) {
        var addons = (this.options.addons || "").split(",");
        var addon = null;
        var self = this;
        var name = this.options.name;
        
        function next(err) {
            if (err)
                return callback(err);
            
            if (addon = addons.shift())
                self.$installAddon(name, addon.trim(), next);
            else
                callback(null, data);
        }
        
        next();
    }
    
    this.create = function(callback) {
        var self = this;
        this.options.stack = this.options.stack || "aspen-mri-1.8.6";
        this.options.timeout = (this.options.timeout || 30) * 1000;
        
        var name = "";
        if (this.options.name)
            name = this.options.name.toLowerCase().trim();
        
        delete this.options.name;
        this.$client.create(name, this.options, function(err, app) {
            if (err)
                return callback(err);
            
            self.options = app;
            /*self.$client.info(app.name, function(err) { // @todo use "info" instead
                if (err)
                    return callback(err);*/
                createComplete.call(self, function(err, data) {
                    if (err)
                        return callback(err);

                    callback(null, data);
                });
            /*});*/
        });
    };
    
    this.destroy = function(callback) {
        var self = this;
        this.info(function(err, data) {
            if (err)
                return callback(err);
            
            self.$client.destroy(data.name, callback);
        });
    };
    
    this.$installAddon = function(addon, callback) {
        var name = this.options.name;
        
        this.$client.installAddon(name, addon, callback);
    };
    
}).call(HerokuCommandApps.prototype);